import mido
import time
from mido import Message
import sys
import os
import itertools

# Open the MIDI output port
def open_midi_port():
    port_name = None
    print("Available MIDI output ports:")
    for name in mido.get_output_names():
        if "Uno MIDI Interface" in name:
            port_name = name
            print("> ", end="")
        print(name)
    # port_name = input("Choose a port: ")

    port = mido.open_output(port_name)
    return port

class KlapkoBřinkoStroj:
    def __init__(self):
        self.port = open_midi_port()  # Open the MIDI port

    def flush_input(self):
        for note in range(128):
            self.port.send(Message('note_off', note=note, velocity=0))

    def send_chord(self, chord, duration, velocity):
        for note in chord:
            self.port.send(Message('note_on', note=note, velocity=velocity))
        time.sleep(duration)  # Duration for which chord is held
        for note in chord:
            self.port.send(Message('note_off', note=note, velocity=velocity))

    def sustain(self, status):
        if status:
            # print("<sus> JOO")
            self.port.send(Message('control_change', control=64, value=127))
        else:
            # print("</sus> NEE")
            self.port.send(Message('control_change', control=64, value=0))

    # def all_combinations(self):
    #     interval = 0.2  # Time interval between chords in seconds
    #     duration = 0.1 # Duration for each chord in seconds

    #     for i in range(0, 12):
    #         for chord in itertools.combinations(range(60, 72), i):
    #             print(chord)
    #             self.send_chord(chord, duration)
    #             time.sleep(interval - duration)
    
    def all_binary(self):
        try:

            interval = 0.25  # Time interval between chords in seconds
            duration = 0.25/4*3  # Duration for each chord in seconds
            main_binary_counter = 1
            while main_binary_counter < 2**12:
                cifry = []
                chord = []
                binary_array = []
                for i in range(0, 12):
                    # j = ((i*5)%12)
                    j = i
                    if main_binary_counter & (1 << i):
                        cifry.append(j)
                        chord.append(60+j)
                        binary_array.insert(0,1)
                    else:
                        binary_array.insert(0,0)

                # sustain
                sustain_period = 32
                if len(chord) == 0 or len(chord) == 1:
                    time.sleep(interval*12)
                    self.sustain(False)
                    print("\n"*8, end="")
                    time.sleep(interval*4)

                    self.sustain(True)
                elif (main_binary_counter % sustain_period == sustain_period-2):
                    self.sustain(False)
                elif (main_binary_counter % sustain_period == 0):
                    self.sustain(True)

                # printing
                # chord_string = repr(chord)
                # chord_string = " ".join(["◯"*(x+1) + " "*(x+1) for x in cifry])
                chord_string = " ".join(["⬤" if x else " " for x in binary_array])
                chord_string = chord_string.lstrip()
                width = os.get_terminal_size().columns
                # print chordstring to the center
                print(chord_string.center(width))

                # intenzita pulzuje každých 16 akordů
                # print("intenzita 16:", (main_binary_counter & 0b1111))
                # print("intenzita 128:", ((main_binary_counter & 0b1110000) >> 4))
                v = 64 + (main_binary_counter & 0b1111) + ((main_binary_counter & 0b1110000) >> 4)
                # v = 64
                self.send_chord(chord, duration, v)
                time.sleep(interval - duration)  # Wait for the next chord

                main_binary_counter += 1
        except KeyboardInterrupt:
            print("Stopped")
            self.flush_input()
            self.port.send(Message('control_change', control=64, value=0))


    def play(self, setting):
            if setting == "all_combinations":
                self.all_combinations()
            elif setting == "all_binary":
                self.all_binary()


# # Function to send a chord (list of notes) to the MIDI device
# def send_chord(port, chord, duration, velocity=64):
#     for note in chord:
#         port.send(Message('note_on', note=note, velocity=velocity))
#     time.sleep(duration)  # Duration for which chord is held
#     for note in chord:
#         port.send(Message('note_off', note=note, velocity=velocity))


# def all_combinations():
#     port = open_midi_port()  # Open the MIDI port

#     interval = 0.6  # Time interval between chords in seconds
#     duration = 0.1 # Duration for each chord in seconds

#     for i in range(0, 12):
#         for chord in itertools.combinations(range(60, 72), i):
#             print(chord)
#             send_chord(port, chord, duration)
#             time.sleep(interval - duration)

# def all_binary():
#     port = open_midi_port()  # Open the MIDI port

#     interval = 0.2  # Time interval between chords in seconds
#     duration = 0.2  # Duration for each chord in seconds

#     chord_binary = 0
#     while chord_binary < 2**12:
#         chord = []
#         for i in range(0, 12):
#             if chord_binary & (1 << i):
#                 chord.append(60+i)
#             # chord.append(1 if  else 0)
#         print(chord)
#         chord_binary += 1

#         send_chord(port, chord, duration)
#         time.sleep(interval - duration)  # Wait for the next chord


# Main loop to send chords at periodic intervals
# def main():
#     # Define the chords (MIDI note numbers)
#     chords = [
#         [60, 64, 67],  # C major
#         [62, 65, 69],  # D minor
#         [64, 67, 71],  # E minor
#         [65, 69, 72],  # F major
#     ]
    
#     interval = 2  # Time interval between chords in seconds
#     duration = 1  # Duration for each chord in seconds

#     port = open_midi_port()  # Open the MIDI port

#     try:
#         while True:
#             for chord in chords:
#                 send_chord(port, chord, duration)
#                 time.sleep(interval - duration)  # Wait for the next chord
#     except KeyboardInterrupt:
#         print("Stopped")

if __name__ == "__main__":
    # Print the size of terminal
    setting = sys.argv[1]
    kbs = KlapkoBřinkoStroj()
    kbs.play(setting)
    # if setting == "all_combinations":
    #     all_combinations()
    # elif setting == "all_binary":
    #     all_binary()

